@@ -16,7 +16,7 @@
  * Host.cpp
  *
  *  Created on: Mar 6, 2017
- *      Author: steven
+ *      Author: Steven Hessing (steven.hessing@gmail.com)
  */
 
 #include <cstring>
@@ -57,8 +57,8 @@ bool Host::Match(const DeviceProfileMap& dpMap) {
 	ConfidenceLevel bestmatch = ConfidenceLevel::None;
 	std::string matcheduuid = "";
 	for (auto &kv : dpMap) {
-		if(Debug) {
-			syslog(LOG_DEBUG, "Evaluating host %s against device profile %s", Mac.c_str(), kv.first.c_str());
+		if(Debug == true) {
+			syslog(LOG_DEBUG, "Host: Evaluating host %s against device profile %s", Mac.c_str(), kv.first.c_str());
 		}
 		auto &dp = *(kv.second);
 		auto match = Match(dp);
@@ -72,7 +72,7 @@ bool Host::Match(const DeviceProfileMap& dpMap) {
 	}
 	if (bestmatch >= ConfidenceLevel::Low) {
 		Uuid = matcheduuid;
-		syslog(LOG_DEBUG, "Host %s matched %s", Mac.c_str(), Uuid.c_str());
+		syslog(LOG_DEBUG, "Host: Host %s matched %s", Mac.c_str(), Uuid.c_str());
 		return true;
 	}
 	return false;
@@ -83,12 +83,12 @@ ConfidenceLevel Host::Match(const DeviceProfile& dp) {
 	auto v = dp.getIdentifiers();
 	for (auto& i : v) {
 		if(Debug) {
-			syslog (LOG_DEBUG, "Testing identifier");
+			syslog (LOG_DEBUG, "Host: Testing identifier");
 		}
 		auto match = Match(*i);
 		if (match >= ConfidenceLevel::High) {
 			if(Debug) {
-				syslog(LOG_DEBUG, "Host %s matched with high confidence", Mac.c_str());
+				syslog(LOG_DEBUG, "Host: Host %s matched with high confidence", Mac.c_str());
 			}
 			return match;
         }
@@ -97,7 +97,7 @@ ConfidenceLevel Host::Match(const DeviceProfile& dp) {
         }
 	}
 	if(Debug) {
-		syslog(LOG_DEBUG, "Host %s match level %d", Mac.c_str(), static_cast<int>(bestmatch));
+		syslog(LOG_DEBUG, "Host: Host %s match level %d", Mac.c_str(), static_cast<int>(bestmatch));
 	}
 	return bestmatch;
 }
@@ -105,28 +105,28 @@ ConfidenceLevel Host::Match(const DeviceProfile& dp) {
 ConfidenceLevel Host::Match(const Identifier& i) {
 	for (auto& mc : i.MatchConditions_get()) {
 		if(Debug) {
-			syslog (LOG_DEBUG, "Testing match condition %s", (*mc).Key.c_str());
+			syslog (LOG_DEBUG, "Host: Testing match condition %s", (*mc).Key.c_str());
 		}
 		if(not Match (*mc)) {
 			if(Debug) {
-				syslog (LOG_DEBUG, "Host %s did not match condition %s", Mac.c_str(), (*mc).Key.c_str());
+				syslog (LOG_DEBUG, "Host: Host %s did not match condition %s", Mac.c_str(), (*mc).Key.c_str());
 			}
 			return ConfidenceLevel::None;
 		}
 	}
 	for (auto& cc : i.ContainConditions_get()) {
 		if(Debug) {
-			syslog (LOG_DEBUG, "Testing contain condition %s", (*cc).Key.c_str());
+			syslog (LOG_DEBUG, "Host: Testing contain condition %s", (*cc).Key.c_str());
 		}
 		if(not Match (*cc)) {
 			if(Debug) {
-				syslog (LOG_DEBUG, "Host %s did not contain condition %s", Mac.c_str(), (*cc).Key.c_str());
+				syslog (LOG_DEBUG, "Host: Host %s did not contain condition %s", Mac.c_str(), (*cc).Key.c_str());
 			}
 			return ConfidenceLevel::None;
 		}
 	}
 	if(Debug) {
-		syslog(LOG_DEBUG, "Host %s matched MustMatch and/or MustContain conditions", Mac.c_str());
+		syslog(LOG_DEBUG, "Host: Host %s matched MustMatch and/or MustContain conditions", Mac.c_str());
 	}
 	return i.IdentifyConfidenceLevel_get();
 }
@@ -137,7 +137,7 @@ bool Host::Match(const MatchCondition& mc) {
 		value = Mac.str().substr(0,8);
 	} else if (mc.Key == "DhcpVendor") {
 		value = Dhcp.DhcpVendor;
-	} else if (mc.Key == "Hostname") {
+	} else if (mc.Key == "DhcpHostname") {
 		value = Dhcp.Hostname ;
 	} else if (mc.Key == "SsdpFriendlyName" ) {
 		value = Ssdp.FriendlyName;
@@ -157,17 +157,31 @@ bool Host::Match(const MatchCondition& mc) {
 		value = Ssdp.Server;
 	} else if (mc.Key == "SsdpLocation") {
 		value = Ssdp.Location;
+	} else if (mc.Key == "WsDiscoveryXAddrs") {
+	    value = Wsd.wsdXAddrs;
+	} else if (mc.Key == "WsDiscoveryTypes") {
+	    value = Wsd.wsdTypes;
+	} else if (mc.Key == "MdnsOs") {
+	    value = Mdns.Os;
+	} else if (mc.Key == "MdnsHw") {
+	    value = Mdns.Hw;
+    } else if (mc.Key == "MdnsDeviceUrl") {
+        value = Mdns.DeviceUrl;
+    } else if (mc.Key == "MdnsManufacturer") {
+        value = Mdns.Manufacturer;
+    } else if (mc.Key == "MdnsModelName") {
+        value = Mdns.ModelName;
 	}
 	if (value == "") {
 		if(Debug) {
-			syslog(LOG_DEBUG, "Host %s has no value for MustMatch condition %s", Mac.c_str(), mc.Key.c_str());
+			syslog(LOG_DEBUG, "Host: Host %s has no value for MustMatch condition %s", Mac.c_str(), mc.Key.c_str());
 		}
 	}
 	size_t startpos = 0;
 	size_t mcvaluelength = mc.Value.length();
 	size_t datavaluelength = value.length();
 	std::string mcvalue = mc.Value;
-	if (mc.Key[0] == '*') {
+	if (mc.Value[0] == '*') {
 		mcvalue = mcvalue.substr(1);
 		startpos = datavaluelength - mcvalue.length();
 	} else if (mc.Value[mcvaluelength-1] == '*') {
@@ -176,12 +190,12 @@ bool Host::Match(const MatchCondition& mc) {
 	}
 	if (value.compare(startpos, mcvalue.length() - startpos, mcvalue) == 0) {
 		if(Debug) {
-			syslog(LOG_DEBUG, "Host %s matched MustMatch condition", Mac.c_str());
+			syslog(LOG_DEBUG, "Host: Host %s matched MustMatch condition", Mac.c_str());
 		}
 		return true;
     }
 	if(Debug) {
-		syslog (LOG_DEBUG, "Host %s did not match condition %s with value %s from position %zu", value.c_str(), mc.Key.c_str(), mcvalue.c_str(), startpos);
+		syslog (LOG_DEBUG, "Host: Host %s did not match condition %s with value %s from position %zu", value.c_str(), mc.Key.c_str(), mcvalue.c_str(), startpos);
 	}
 	return false;
 }
@@ -191,23 +205,23 @@ bool Host::Match(const ContainCondition& cc) {
 		for (auto fqdn: cc.Values) {
 		    if (DnsQueryList.find(fqdn) != DnsQueryList.end()) {
 				if(Debug) {
-					syslog(LOG_DEBUG, "Found DnsQuery for %s from host %s", fqdn.c_str(), Mac.c_str());
+					syslog(LOG_DEBUG, "Host: Found DnsQuery for %s from host %s", fqdn.c_str(), Mac.c_str());
 				}
 			} else {
 				if(Debug) {
-					syslog(LOG_DEBUG, "Didn't find DnsQuery for %s from host %s", fqdn.c_str(), Mac.c_str());
+					syslog(LOG_DEBUG, "Host: Didn't find DnsQuery for %s from host %s", fqdn.c_str(), Mac.c_str());
 				}
 				return false;
 			}
 		}
 	} else {
 		if(Debug) {
-			syslog(LOG_DEBUG, "Unsupported MustContain key %s", cc.Key.c_str());
+			syslog(LOG_DEBUG, "Host: Unsupported MustContain key %s", cc.Key.c_str());
 		}
 		return false;
 	}
 	if(Debug) {
-		syslog(LOG_DEBUG, "Host %s matched MustContain condition", Mac.c_str());
+		syslog(LOG_DEBUG, "Host: Host %s matched MustContain condition", Mac.c_str());
 	}
 	return true;
 }
@@ -220,7 +234,7 @@ bool Host::DeviceStats(json& j, const uint32_t time_interval, bool force, bool d
 	}
 	j["MacOid"] = Mac.str().substr(0,8);
 	j["DhcpVendor"] = Dhcp.DhcpVendor;
-	j["Hostname"] = Dhcp.Hostname;
+	j["DhcpHostname"] = Dhcp.Hostname;
 	j["SsdpFriendlyName"] = Ssdp.FriendlyName;
 	j["SsdpManufacturer"] = Ssdp.Manufacturer;
 	j["SsdpManufacturerUrl"] = Ssdp.ManufacturerUrl;
@@ -229,16 +243,46 @@ bool Host::DeviceStats(json& j, const uint32_t time_interval, bool force, bool d
 	j["SsdpSerialNumber"] = Ssdp.SerialNumber;
 	j["SsdpUserAgent"] = Ssdp.UserAgent;
 	j["SsdpServer"] = Ssdp.Server;
-	j["SsdpLocation"] = Ssdp.Location;
+    j["WsDiscoveryXaddrs"] = Wsd.wsdXAddrs;
+    j["WsDiscoveryTypes"] = Wsd.wsdTypes;
+    j["MdnsOs"] = Mdns.Os;
+    j["MdnsHw"] = Mdns.Hw;
+    j["MdnsDeviceUrl"] = Mdns.DeviceUrl;
+    j["MdnsManufacturer"] = Mdns.Manufacturer;
+    j["MdnsModelName"] = Mdns.ModelName;
+
+    // We don't want to upload any IP addresses, even if they are RFC1918 so we replace them with 'a.b.c.d'
+    if (Ssdp.Location != "" || Mdns.DeviceUrl != "") {
+        std::regex location_rx = std::regex(R"delim((https?://)\d+\.\d+\.\d+\.\d+(.*)$)delim",
+                std::regex_constants::ECMAScript | std::regex_constants::icase | std::regex_constants::optimize);
+        std::string location = Ssdp.Location;
+        if (Ssdp.Location != "") {
+            std::smatch m;
+            std::regex_search(Ssdp.Location, m, location_rx);
+            if (not m.empty()) {
+                location = m.str(1) + "a.b.c.d" + m.str(2);
+            }
+        }
+        j["SsdpLocation"] = location;
+        location = Mdns.DeviceUrl;
+        if (Mdns.DeviceUrl != "") {
+            std::smatch m;
+            std::regex_search(Mdns.DeviceUrl, m, location_rx);
+            if (not m.empty()) {
+                location = m.str(1) + "a.b.c.d" + m.str(2);
+            }
+        }
+        j["MdnsDeviceUrl"] = location;
+    }
 
 	std::string fqdns = "";
 	if (Debug == true) {
-		syslog (LOG_DEBUG, "Adding items from DnsQueryList to list of fqdns");
+		syslog (LOG_DEBUG, "Host: Adding items from DnsQueryList to list of fqdns");
 	}
 	for (auto &dq: DnsQueryList) {
 		if (detailed == false) {
 			if (Debug == true) {
-				syslog (LOG_DEBUG, "Adding FQDN %s from DnsQueryList to list of fqdns", dq.first.c_str());
+				syslog (LOG_DEBUG, "Host: Adding FQDN %s from DnsQueryList to list of fqdns", dq.first.c_str());
 			}
 			fqdns += dq.first + " ";
 		}
@@ -251,8 +295,9 @@ bool Host::DeviceStats(json& j, const uint32_t time_interval, bool force, bool d
 	return true;
 }
 
-bool Host::TrafficStats(json& j, const uint32_t interval, const bool ReportPrivateAddresses, const std::unordered_set<std::string> &LocalIps,
-		const DnsIpCache <boost::asio::ip::address> &dCip, const DnsCnameCache &dCcname, bool force) {
+bool Host::TrafficStats(json& j, const uint32_t interval, const bool ReportPrivateAddresses, const std::set<std::string> &LocalIps,
+		const DnsIpCache <Tins::IPv4Address> &dCipv4, const DnsIpCache <Tins::IPv6Address> &dCipv6,
+		const DnsCnameCache &dCcname, bool force) {
 	if (not isMatched()) {
 		return false;
 	}
@@ -263,34 +308,69 @@ bool Host::TrafficStats(json& j, const uint32_t interval, const bool ReportPriva
 	std::map<std::string,std::shared_ptr<std::unordered_set<std::string>>> allIps;
 
 	std::unordered_set<std::string> endpoints;
-	for (auto &fc: FlowCacheIpv4) {
-		const boost::asio::ip::address_v4  &ip = fc.first;
-		if (ReportPrivateAddresses || not inPrivateAddressRange(ip.to_string())) {
-			auto it = allIps.find(ip.to_string());
-			if (it != allIps.end()) {
-				for (auto &fqdn: *(it->second)) {
-					endpoints.insert(fqdn);
-				}
-			}
-			if (Debug == true) {
-				syslog (LOG_DEBUG, "Getting all DNS lookups for %s", ip.to_string().c_str());
-			}
-			std::vector<std::string> fqdns = dCip.getAllFqdns(ip);
-			for (auto &itf : fqdns) {
-				std::string fqdn = dCcname.resolveCname(itf);
-                if (Debug) {
-                    syslog (LOG_DEBUG, "Reverse resolved %s to %s, might have CNAME %s", ip.to_string().c_str(), itf.c_str(), fqdn.c_str());
-                }
-				if (inDnsQueryList(fqdn)) {
-				    endpoints.insert(fqdn);
-				}
-			}
-			if (endpoints.size() == 0) {
-				endpoints.insert(ip.to_string());
-			}
-		}
+	{
+	    for (auto &fc: FlowCacheIpv4) {
+	        bool foundFqdn = false;
+	        Tins::IPv4Address  ip(fc.first);
+	        if (ReportPrivateAddresses == true || not ip.is_private()) {
+	            auto it = allIps.find(ip.to_string());
+	            if (it != allIps.end()) {
+	                for (auto &fqdn: *(it->second)) {
+	                    endpoints.insert(fqdn);
+	                    foundFqdn = true;
+	                }
+	            }
+	            if (Debug == true) {
+	                syslog (LOG_DEBUG, "Host: Getting all DNS lookups for %s", ip.to_string().c_str());
+	            }
+	            std::vector<std::string> fqdns = dCipv4.getAllFqdns(ip);
+	            for (auto &itf : fqdns) {
+	                std::string fqdn = dCcname.getFqdn(itf);
+	                if (Debug) {
+	                    syslog (LOG_DEBUG, "Host: Reverse resolved %s to %s, might have CNAME %s", ip.to_string().c_str(), itf.c_str(), fqdn.c_str());
+	                }
+	                if (inDnsQueryList(fqdn)) {
+	                    endpoints.insert(fqdn);
+	                    foundFqdn = true;
+	                }
+	            }
+	            if (foundFqdn) {
+	                endpoints.insert(ip.to_string());
+	            }
+	        }
+	    }
 	}
+    {
+        for (auto &fc: FlowCacheIpv6) {
+            bool foundFqdn = false;
+            const Tins::IPv6Address  &ip = fc.first;
+            auto it = allIps.find(ip.to_string());
+            if (it != allIps.end()) {
+                for (auto &fqdn: *(it->second)) {
+                    endpoints.insert(fqdn);
+                    foundFqdn = true;
+                }
+            }
+            if (Debug == true) {
+                syslog (LOG_DEBUG, "Host: Getting all DNS lookups for %s", ip.to_string().c_str());
+            }
+            std::vector<std::string> fqdns = dCipv6.getAllFqdns(ip);
+            for (auto &itf : fqdns) {
+                std::string fqdn = dCcname.getFqdn(itf);
+                if (Debug) {
+                    syslog (LOG_DEBUG, "Host: Reverse resolved %s to %s, might have CNAME %s", ip.to_string().c_str(), itf.c_str(), fqdn.c_str());
+                }
+                if (inDnsQueryList(fqdn)) {
+                    endpoints.insert(fqdn);
+                    foundFqdn = true;
+                }
+            }
+            if (foundFqdn == true) {
+                endpoints.insert(ip.to_string());
+            }
 
+        }
+    }
 	if (endpoints.size() > 0) {
 		j = { {"DeviceProfileUuid", Uuid } };
 		j["TrafficStats"] = endpoints;
@@ -328,6 +408,7 @@ void Host::ExportDeviceInfo (json &j, bool detailed) {
 	json h{{"MacAddress", Mac.str()},
 			{"DeviceProfileUuid", Uuid},
 			{"Ipv4Address", Ipv4Address},
+			{"Ipv6Address", Ipv6Address},
 			{"SsdpManufacturer", Ssdp.Manufacturer},
 			{"SsdpModelName", Ssdp.ModelName}
 	};
@@ -342,18 +423,18 @@ void Host::ExportDeviceInfo (json &j, bool detailed) {
  *  Adds or updates the list of flows from a host to a remote host
  *  Returns true if flow was added, false if existing flow was updated
  */
-bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
+bool Host::setFlowEntry(const uint16_t inSrcPort, const std::string inDstIp,
 			const uint16_t inDstPort, const uint8_t inProtocol, const uint32_t inExpiration) {
 	iCache::LastSeen = time(nullptr);
 	if (inDstIp == "239.255.255.250") {
 		if(Debug) {
-			syslog(LOG_DEBUG, "Ignoring flow to 239.255.255.0");
+			syslog(LOG_DEBUG, "Host: Ignoring flow to 239.255.255.0");
 		}
 		return false;
 	}
 	auto f = std::make_shared<FlowEntry>();
 	if(Debug) {
-		syslog(LOG_DEBUG, "Creating new Flow Entry for src port %u, dest ip %s, dest port %u, protocol %u", inSrcPort,
+		syslog(LOG_DEBUG, "Host: Creating new Flow Entry for src port %u, dest ip %s, dest port %u, protocol %u", inSrcPort,
 				inDstIp.c_str(), inDstPort, inProtocol);
 	}
 	iCache::LastModified = time(nullptr);
@@ -362,17 +443,14 @@ bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
 	f->Protocol = inProtocol;
 	f->setExpiration(inExpiration);
 
-	boost::asio::ip::address dstIpAddress = boost::asio::ip::address::from_string(inDstIp);
-
-	// Is there already at least one flow from the Host to the destination IP?
-	if (dstIpAddress.is_v4()) {
-		boost::asio::ip::address_v4 dstIpv4Address = dstIpAddress.to_v4();
+	try {
+		Tins::IPv4Address dstIpv4Address(inDstIp);
 
 		if (FlowCacheIpv4.find(dstIpv4Address) == FlowCacheIpv4.end()) {
 			FlowCacheIpv4[dstIpv4Address] = std::make_shared<FlowEntryList>();
 			FlowCacheIpv4[dstIpv4Address]->push_back(f);
 			if(Debug) {
-				syslog(LOG_DEBUG, "Adding to IPv4 FlowCache with destination %s : %u Protocol %u", inDstIp.c_str(),
+				syslog(LOG_DEBUG, "Host: Adding to IPv4 FlowCache with destination %s : %u Protocol %u", inDstIp.c_str(),
 					inDstPort, inProtocol);
 			}
 			return true;
@@ -383,7 +461,7 @@ bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
 			// Update existing flow it it matches incoming flow (ignoring Expiration)
 			if (**existingflow == *f) {
 				if(Debug) {
-					syslog(LOG_DEBUG, "Updating expiration of existing FlowEntry in IPv4 FlowCache for destination %s",
+					syslog(LOG_DEBUG, "Host: Updating expiration of existing FlowEntry in IPv4 FlowCache for destination %s",
 						inDstIp.c_str());
 				}
 				(*existingflow)->setExpiration(inExpiration);
@@ -392,18 +470,18 @@ bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
 		}
 		// This flow doesn't match any of the existing flows
 		if(Debug) {
-			syslog(LOG_DEBUG, "Adding FlowEntry to IPv4 FlowCache for destination %s", inDstIp.c_str());
+			syslog(LOG_DEBUG, "Host: Adding FlowEntry to IPv4 FlowCache for destination %s", inDstIp.c_str());
 		}
 		FlowCacheIpv4[dstIpv4Address]->push_back(f);
 		return true;
-	} else if (dstIpAddress.is_v6()) {
-		boost::asio::ip::address_v6 dstIpv6Address = dstIpAddress.to_v6();
+	} catch (...) {
+		Tins::IPv6Address dstIpv6Address(inDstIp);
 
 		if (FlowCacheIpv6.find(dstIpv6Address) == FlowCacheIpv6.end()) {
 			FlowCacheIpv6[dstIpv6Address] = std::make_shared<FlowEntryList>();
 			FlowCacheIpv6[dstIpv6Address]->push_back(f);
 			if(Debug) {
-				syslog(LOG_DEBUG, "Adding to IPv6 FlowCache with destination %s : %u Protocol %u", inDstIp.c_str(),
+				syslog(LOG_DEBUG, "Host: Adding to IPv6 FlowCache with destination %s : %u Protocol %u", inDstIp.c_str(),
 					inDstPort, inProtocol);
 			}
 			return true;
@@ -414,7 +492,7 @@ bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
 			// Update existing flow it it matches incoming flow (ignoring Expiration)
 			if (**existingflow == *f) {
 				if(Debug) {
-					syslog(LOG_DEBUG, "Updating expiration of existing FlowEntry in IPv6 FlowCache for destination %s",
+					syslog(LOG_DEBUG, "Host: Updating expiration of existing FlowEntry in IPv6 FlowCache for destination %s",
 						inDstIp.c_str());
 				}
 				(*existingflow)->setExpiration(inExpiration);
@@ -423,16 +501,16 @@ bool Host::FlowEntry_set(const uint16_t inSrcPort, const std::string inDstIp,
 		}
 		// This flow doesn't match any of the existing flows
 		if(Debug) {
-			syslog(LOG_DEBUG, "Adding FlowEntry to IPv6 FlowCache for destination %s", inDstIp.c_str());
+			syslog(LOG_DEBUG, "Host: Adding FlowEntry to IPv6 FlowCache for destination %s", inDstIp.c_str());
 		}
 		FlowCacheIpv6[dstIpv6Address]->push_back(f);
 		return true;
 	}
-	syslog(LOG_NOTICE, "IP address %s is neither v4 or v6", inDstIp.c_str());
+	syslog(LOG_NOTICE, "Host: IP address %s is neither v4 or v6", inDstIp.c_str());
 	return false;
 }
 
-bool Host::Dhcp_set (const std::string inIpAddress, const MacAddress inMac, const std::string inHostname, const std::string inDhcpVendor) {
+bool Host::setDhcp (const std::string inIpAddress, const MacAddress inMac, const std::string inHostname, const std::string inDhcpVendor) {
     if (not inMac.isValid()) {
             return false;
     }
@@ -450,20 +528,26 @@ bool Host::Dhcp_set (const std::string inIpAddress, const MacAddress inMac, cons
 	iCache::FirstSeen = iCache::LastModified = iCache::LastSeen = time(nullptr);
 	Dhcp.setExpiration();
 	if(Debug) {
-		syslog(LOG_DEBUG, "Creating DHCP data for %s with expiration %lu with ipaddress %s, hostname %s, vendor %s ",
+		syslog(LOG_DEBUG, "Host: Creating DHCP data for %s with expiration %lu with ipaddress %s, hostname %s, vendor %s ",
 		        Dhcp.Mac.c_str(), Dhcp.getExpiration(), Dhcp.IpAddress.c_str(), Dhcp.Hostname.c_str(), Dhcp.DhcpVendor.c_str());
 	}
 	return true;
 }
 
-bool Host::SsdpInfo_set(const std::shared_ptr<SsdpHost> insHost) {
+bool Host::setSsdpInfo(const std::shared_ptr<SsdpHost> insHost) {
 	iCache::LastSeen = time(nullptr);
 	if (Ssdp == *insHost) {
-		return false;
+        if (Debug == true) {
+            syslog (LOG_DEBUG, "Host: not setting Ssdp info as same info is already known");
+        }
+        return false;
 	}
 
 	iCache::LastModified = iCache::LastSeen;
 	Ssdp = *insHost;
+    if (Debug == true) {
+        syslog (LOG_DEBUG, "Host: SSDP info has been set");
+    }
 
 	// Information in the SSDP multicast message has changed so if the Location field contains a URL, we query it
 	if (Ssdp.Location != "") {
@@ -472,6 +556,38 @@ bool Host::SsdpInfo_set(const std::shared_ptr<SsdpHost> insHost) {
 	return true;
 }
 
+bool Host::setWsDiscoveryInfo(const std::shared_ptr<WsDiscoveryHost> inwsdHost) {
+    iCache::LastSeen = time(nullptr);
+    if (Wsd == *inwsdHost) {
+        if (Debug == true) {
+            syslog (LOG_DEBUG, "Host: not setting WS-Discovery info as same info is already known");
+        }
+        return false;
+    }
+    iCache::LastModified = iCache::LastSeen;
+    Wsd = *inwsdHost;
+    if (Debug == true) {
+        syslog (LOG_DEBUG, "Host: WS-Discovery info has been set");
+    }
+    return true;
+}
+
+bool Host::setMdnsInfo(const std::shared_ptr<MdnsHost> inmdnsHost) {
+    iCache::LastSeen = time(nullptr);
+    if (Mdns == *inmdnsHost) {
+        if (Debug == true) {
+            syslog (LOG_DEBUG, "Host: not setting mDNS info as same info is already known");
+        }
+        return false;
+    }
+    iCache::LastModified = iCache::LastSeen;
+    Mdns = *inmdnsHost;
+    if (Debug == true) {
+        syslog (LOG_DEBUG, "Host: mDNS info has been set");
+    }
+    return true;
+}
+
 bool Host::UploadsEnabled() {
 	return UploadStats;
 }
@@ -479,7 +595,7 @@ bool Host::UploadsEnabled() {
 uint32_t Host::Prune (bool Force) {
 	bool pruned = false;
 	if(Debug) {
-		syslog(LOG_DEBUG, "Pruning host %s", Mac.c_str());
+		syslog(LOG_DEBUG, "Host: Pruning host %s", Mac.c_str());
 	}
 	pruneDnsQueryList(Force);
 
@@ -501,7 +617,7 @@ uint32_t Host::Prune (bool Force) {
 				if (Force || (*it)->isExpired()) {
 					if (Debug) {
 						std::string dstIp = (fc->first).to_string();
-						syslog(LOG_DEBUG, "Pruning IPv4 FlowEntry to %s for DstPort %u with expiration %ld while now is %ld",
+						syslog(LOG_DEBUG, "Host: Pruning IPv4 FlowEntry to %s for DstPort %u with expiration %ld while now is %ld",
 								dstIp.c_str(), (*it)->DstPort, (*it)->getExpiration (), time(nullptr));
 					}
 					// Remove element from list
@@ -515,7 +631,7 @@ uint32_t Host::Prune (bool Force) {
 			// If the list of Flow Entry pointers is empty, delete it
 			if (Force || fel.empty()) {
 				if (Debug) {
-					syslog(LOG_DEBUG, "Pruning FlowEntryList for %s as it is now empty", fc->first.to_string().c_str());
+					syslog(LOG_DEBUG, "Host: Pruning FlowEntryList for %s as it is now empty", fc->first.to_string().c_str());
 				}
 				fc = FlowCacheIpv4.erase(fc);
 				pruned = true;
@@ -538,7 +654,7 @@ uint32_t Host::Prune (bool Force) {
 			while (it != fel.end()) {
 				if (Force || (*it)->isExpired()) {
 					if (Debug) {
-						syslog(LOG_DEBUG, "Pruning IPv6 FlowEntry to %s for DstPort %u with expiration %ld while now is %ld",
+						syslog(LOG_DEBUG, "Host: Pruning IPv6 FlowEntry to %s for DstPort %u with expiration %ld while now is %ld",
 								fc->first.to_string().c_str(), (*it)->DstPort, (*it)->getExpiration (), time(nullptr));
 					}
 					// Remove element from list
@@ -552,7 +668,7 @@ uint32_t Host::Prune (bool Force) {
 			// If the list of Flow Entry pointers is empty, delete it
 			if (Force || fel.empty()) {
 				if (Debug) {
-					syslog(LOG_DEBUG, "Pruning FlowEntryList for %s as it is now empty", fc->first.to_string().c_str());
+					syslog(LOG_DEBUG, "Host: Pruning FlowEntryList for %s as it is now empty", fc->first.to_string().c_str());
 				}
 				fc = FlowCacheIpv6.erase(fc);
 				pruned = true;
@@ -562,7 +678,7 @@ uint32_t Host::Prune (bool Force) {
 			}
 		}
 		if(Debug) {
-			syslog (LOG_DEBUG, "Pruned %u Flow Entries and %u flows", pruned_flowentries, pruned_flows);
+			syslog (LOG_DEBUG, "Host: Pruned %u Flow Entries and %u flows", pruned_flowentries, pruned_flows);
 		}
 	}
 	return pruned;
@@ -575,7 +691,7 @@ uint32_t Host::pruneDnsQueryList (time_t Expired, bool Force) {
 	while (i != DnsQueryList.end()) {
 		if (Force || i->second > (now - Expired)) {
 			if (Debug == true) {
-				syslog (LOG_DEBUG, "Deleting %s from DnsQueryList as %lu is later than %lu", i->first.c_str(), i->second, now - Expired);
+				syslog (LOG_DEBUG, "Host: Deleting %s from DnsQueryList as %lu is later than %lu", i->first.c_str(), i->second, now - Expired);
 			}
 			i = DnsQueryList.erase(i);
 			deletecount++;
